import 'dart:convert';

import 'package:microsoft_graph/src/core/http_client.dart';
import 'package:microsoft_graph/src/core/http_method.dart';
import 'package:microsoft_graph/src/core/http_request_message.dart';
import 'package:microsoft_graph/src/models/folder_id.dart';
import 'package:microsoft_graph/src/models/message.dart';

final _endpoint = "https://graph.microsoft.com/v1.0/me";

Stream<List<Message>> getNotes(IHttpClient httpClient, FolderId folderId) async* {
  var pageUrl;
  if (folderId is WellKnownFolderId) {
    pageUrl = "$_endpoint/mailFolders/${folderId.name}/messages";
  } else if (folderId is SpecificFolderId) {
    pageUrl = "$_endpoint/mailFolders/${folderId.id}/messages";
  } else {
    throw Exception("Unknown folderId: $folderId");
  }

  while(true) {
    final request = HttpRequestMessage(HttpMethod.Get, Uri.parse(pageUrl), []);
    final authenticatedRequest = await httpClient.authenticationProvider.authenticateRequest(request);
    final response = await httpClient.httpProvider.send(authenticatedRequest);

    Map<String, dynamic> json = jsonDecode(response.body);
    List<Message> notes = new List();
    for (int i = 0; i < json["value"].length; i++) {
      var note = Message(
          json["value"][i]["id"],
          json["value"][i]["subject"] ?? "",
          json["value"][i]["bodyPreview"] ?? "",
          json["value"][i]["body"]["content"] ?? "",
          DateTime.parse(json["value"][i]["lastModifiedDateTime"]));
      notes.add(note);
    }
    yield notes;
    if (json["@odata.nextLink"] == null) {
      return;
    } else {
      pageUrl = json["@odata.nextLink"];
    }
  }
}